// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const GMenu = imports.gi.GMenu;
const Shell = imports.gi.Shell;

const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;
const PanelTools = Me.imports.panelTools;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

const KEY_SHOW_APPLICATIONS = 'show-applications';

const ApplicationsButton = new Lang.Class({
    Name: 'ApplicationsButton',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(1.0, _("Applications"), { icon_name: 'start-here-symbolic', text: _("Applications") });
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_APPLICATIONS, Lang.bind(this, this._onShowApplications));
        this._onShowApplications();
        this._appSystem = Shell.AppSystem.get_default();
        this._connectAppSystem = this._appSystem.connect('installed-changed', Lang.bind(this, this._onInstalledChanged));
        this._onInstalledChanged();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._appSystem.disconnect(this._connectAppSystem);
            this._settings.disconnect(this._connectSettings);
        }));
    },

    _getCategory: function(appList, appDir) {
        let iter = appDir.iter();
        let nextType;
        while ((nextType = iter.next()) != GMenu.TreeItemType.INVALID) {
            if (nextType == GMenu.TreeItemType.ENTRY) {
                let entry = iter.get_entry();
                let app = this._appSystem.lookup_app(entry.get_desktop_file_id());
                if (entry.get_app_info().should_show()) appList.push(app);
            } else if (nextType == GMenu.TreeItemType.DIRECTORY) {
                if (!appDir.get_is_nodisplay()) this._getCategory(appList, iter.get_directory());
            }
        }
    },

    _onInstalledChanged: function() {
        let tree = new GMenu.Tree({ menu_basename: 'applications.menu' });
        tree.load_sync();
        let root = tree.get_root_directory();
        let iter = root.iter();
        let nextType;
        this.menu.removeAll();
        while ((nextType = iter.next()) != GMenu.TreeItemType.INVALID) {
            switch (nextType) {
                case GMenu.TreeItemType.SEPARATOR:
                    this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
                    break;
                case GMenu.TreeItemType.ENTRY:
                    let entry = iter.get_entry();
                    let app = this._appSystem.lookup_app(entry.get_desktop_file_id());
                    if (entry.get_app_info().should_show()) this.menu.addMenuItem(new PanelTools.PopupApplicationMenuItem(app, true));
                    break;
                case GMenu.TreeItemType.DIRECTORY:
                    let appDir = iter.get_directory();
                    if (appDir.get_is_nodisplay()) continue;
                    let item = new PanelTools.PopupSubMenuMenuItem(appDir.get_name(), true);
                    if (appDir.get_icon) item.setIconName(appDir.get_icon().get_names().toString());
                    let appList = [];
                    this._getCategory(appList, appDir);
                    appList.sort(function(a, b) {
                        return a.compare_by_name(b);
                    });
                    for (let i = 0; i < appList.length; i++) {
                        item.menu.addMenuItem(new PanelTools.PopupApplicationMenuItem(appList[i], true));
                    }
                    if (item.menu.length < 1) continue;
                    this.menu.addMenuItem(item);
                    break;
            }
        }
    },

    _onShowApplications: function() {
        this.updateButton(this._settings.get_enum(KEY_SHOW_APPLICATIONS));
        PanelTools.updatePanelCorner(this, 'left');
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfApplications', new ApplicationsButton(), 1, 'left');
}
