// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

function _getDateMenu() {
    if (Convenience.checkGSV(3, 4)) {
        try { return Main.panel._dateMenu; } catch(e) { return null; }
    } else {
        try { return Main.panel.statusArea.dateMenu; } catch(e) { return null; }
    }
}

const GNOME_DATEMENU = _getDateMenu();

const KEY_SHOW_DATEMENU = 'show-datemenu';

const DateMenuButton = new Lang.Class({
    Name: 'DateMenuButton',

    _init: function() {
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_DATEMENU, Lang.bind(this, this._onShowDateMenu));
        this._onShowDateMenu();
    },

    _onShowDateMenu: function() {
        this._updateVisibility(this._settings.get_boolean(KEY_SHOW_DATEMENU));
    },

    _updateVisibility: function(visible) {
        GNOME_DATEMENU.actor.visible = visible;
        if (GNOME_DATEMENU.container) GNOME_DATEMENU.container.visible = visible;
    },

    destroy: function() {
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility(true);
    }
});

function getExtension() {
    return GNOME_DATEMENU ? new DateMenuButton() : null;
}
