// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

function _getKeyboard() {
    if (Convenience.checkGSV(3, 4)) {
        try { return Main.panel._statusArea.keyboard; } catch(e) { return null; }
    } else {
        try { return Main.panel.statusArea.keyboard; } catch(e) { return null; }
    }
}

const GNOME_KEYBOARD = _getKeyboard();

const KEY_SHOW_KEYBOARD = 'show-keyboard';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;

const KeyboardIndicator = new Lang.Class({
    Name: Convenience.checkGSV(3, 4) ? 'XKBIndicator' : 'InputSourceIndicator',

    _init: function() {
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_KEYBOARD, Lang.bind(this, this._onShowKeyboard));
        if (Convenience.checkGSV(3, 4)) {
            this._connectKeyboardConfig = GNOME_KEYBOARD._config.connect('changed', Lang.bind(this, this._onShowKeyboard));
        } else if (Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8) || Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12)) {
            this._connectKeyboardSettings = GNOME_KEYBOARD._settings.connect('changed::current', Lang.bind(this, this._onShowKeyboard));
            if (Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8) || Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12)) {
                this._connectKeyboardIbusManager = [
                    GNOME_KEYBOARD._ibusManager.connect('properties-registered', Lang.bind(this, this._onShowKeyboard)),
                    GNOME_KEYBOARD._ibusManager.connect('property-updated', Lang.bind(this, this._onShowKeyboard))
                ];
            }
        } else {
            this._connectKeyboardInputSourceManager = GNOME_KEYBOARD._inputSourceManager.connect('current-source-changed', Lang.bind(this, this._onShowKeyboard));
        }
        this._onShowKeyboard();
    },

    _onShowKeyboard: function() {
        this._updateVisibility(this._settings.get_enum(KEY_SHOW_KEYBOARD));
    },

    _updateVisibility: function(showKeyboard) {
        if (Convenience.checkGSV(3, 4)) {
            GNOME_KEYBOARD._syncConfig();
        } else if (Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8) || Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12)) {
            GNOME_KEYBOARD._currentInputSourceChanged();
        } else {
            GNOME_KEYBOARD._currentSourceChanged();
        }
        if (showKeyboard != PANELMENU_BUTTON_DEFAULT) {
            let inputSources;
            if (Convenience.checkGSV(3, 4)) {
                inputSources = GNOME_KEYBOARD._config.get_group_names();
            } else if (Convenience.checkGSV(3, 6)) {
                inputSources = Object.keys(GNOME_KEYBOARD._layoutItems);
            } else if (Convenience.checkGSV(3, 8) || Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12)) {
                inputSources = Object.keys(GNOME_KEYBOARD._inputSources);
            } else {
                inputSources = Object.keys(GNOME_KEYBOARD._inputSourceManager.inputSources);
            }
            GNOME_KEYBOARD.actor.visible = inputSources.length > 1 && showKeyboard == PANELMENU_BUTTON_SHOW;
        }
    },

    destroy: function() {
        if (Convenience.checkGSV(3, 4)) {
            GNOME_KEYBOARD._config.disconnect(this._connectKeyboardConfig);
        } else if (Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8) || Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12)) {
            GNOME_KEYBOARD._settings.disconnect(this._connectKeyboardSettings);
            if (Convenience.checkGSV(3, 8) || Convenience.checkGSV(3, 10) || Convenience.checkGSV(3, 12)) {
                for each(let connection in this._connectKeyboardSettings) {
                    GNOME_KEYBOARD._settings.disconnect(connection);
                }
            }
        } else {
            GNOME_KEYBOARD._inputSourceManager.disconnect(this._connectKeyboardInputSourceManager);
        }
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility(PANELMENU_BUTTON_DEFAULT);
    }
});

function getExtension() {
    return GNOME_KEYBOARD ? new KeyboardIndicator() : null;
}
