// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

function _getNetwork() {
    if (Convenience.checkGSV(3, 4)) {
        try { return Main.panel._statusArea.network; } catch(e) { return null; }
    } else if (Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
        try { return Main.panel.statusArea.network; } catch(e) { return null; }
    } else {
        try { return Main.panel.statusArea.aggregateMenu._network; } catch(e) { return null; }
    }
}

const GNOME_NETWORK = _getNetwork();

const KEY_SHOW_NETWORK = 'show-network';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;

let _prototypeSyncNMState;
let _prototypeUpdateIcon;

function _overrideFunction(enabled) {
    if (enabled) {
        _prototypeSyncNMState = imports.ui.status.network.NMApplet.prototype._syncNMState;
        GNOME_NETWORK._syncNMState = function() {
            if (!this._client) return;
            _prototypeSyncNMState.call(this);
            let pmShowNetwork = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_NETWORK);
            if (pmShowNetwork != PANELMENU_BUTTON_DEFAULT) {
                let pmVisible = pmShowNetwork == PANELMENU_BUTTON_SHOW;
                if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
                    if (this.mainIcon) this.mainIcon.visible = pmVisible;
                    this.actor.visible = pmVisible;
                } else {
                    this.indicators.visible = pmVisible;
                    this.menu.actor.visible = pmVisible;
                }
            }
            this._updateIcon();
        };
        _prototypeUpdateIcon = imports.ui.status.network.NMApplet.prototype._updateIcon;
        GNOME_NETWORK._updateIcon = function() {
            _prototypeUpdateIcon.call(this);
            let pmShowNetwork = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_NETWORK);
            if (pmShowNetwork != PANELMENU_BUTTON_DEFAULT) {
                let pmVisible = pmShowNetwork == PANELMENU_BUTTON_SHOW;
                if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
                } else {
                    if (!this._client.networking_enabled || !this._mainConnection || !this._primaryIndicator.visible) {
                        this._primaryIndicator.icon_name = 'network-error-symbolic';
                    } else if (!this._primaryIndicator.icon_name) {
                        this._primaryIndicator.icon_name = 'network-offline-symbolic';
                    }
                    this._primaryIndicator.visible = this._primaryIndicator.icon_name && pmVisible;
                    this._vpnIndicator.visible = this._vpnIndicator.icon_name && pmVisible;
                    this._vpnSection.item.actor.visible = this._vpnIndicator.visible;
                }
            }
        };
    } else {
        GNOME_NETWORK._updateIcon = _prototypeUpdateIcon;
        GNOME_NETWORK._syncNMState = _prototypeSyncNMState;
    }
}

const NMApplet = new Lang.Class({
    Name: 'NMApplet',

    _init: function() {
        _overrideFunction(true);
        this._settings =  Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_NETWORK, Lang.bind(this, this._updateVisibility));
        this._updateVisibility();
    },

    _updateVisibility: function() {
        GNOME_NETWORK._syncNMState();
    },

    destroy: function() {
        _overrideFunction(false);
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility();
        GNOME_NETWORK._updateIcon();
    }
});

function getExtension() {
    return GNOME_NETWORK ? new NMApplet() : null;
}
